@@ -40,16 +40,16 @@ from FreeCAD import Units
 
 from femguiutils import selection_widgets
 from femobjects import element_fluid1D
+from . import base_femtaskpanel
 
 
-class _TaskPanel:
+class _TaskPanel(base_femtaskpanel._BaseTaskPanel):
     """
     The TaskPanel for editing References property of ElementFluid1D objects
     """
 
     def __init__(self, obj):
-
-        self.obj = obj
+        super().__init__(obj)
 
         # parameter widget
         self.parameterWidget = FreeCADGui.PySideUic.loadUi(
@@ -58,162 +58,162 @@ class _TaskPanel:
         QtCore.QObject.connect(
             self.parameterWidget.cb_section_type,
             QtCore.SIGNAL("activated(int)"),
-            self.sectiontype_changed
+            self.sectiontype_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.cb_liquid_section_type,
             QtCore.SIGNAL("activated(int)"),
-            self.liquidsectiontype_changed
+            self.liquidsectiontype_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_manning_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.manning_area_changed
+            self.manning_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_manning_radius,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.manning_radius_changed
+            self.manning_radius_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.sb_manning_coefficient,
             QtCore.SIGNAL("valueChanged(double)"),
-            self.manning_coefficient_changed
+            self.manning_coefficient_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_enlarge_area1,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.enlarge_area1_changed
+            self.enlarge_area1_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_enlarge_area2,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.enlarge_area2_changed
+            self.enlarge_area2_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_contract_area1,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.contract_area1_changed
+            self.contract_area1_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_contract_area2,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.contract_area2_changed
+            self.contract_area2_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_inletpressure,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.inlet_pressure_changed
+            self.inlet_pressure_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_outletpressure,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.outlet_pressure_changed
+            self.outlet_pressure_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_inletflowrate,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.inlet_flowrate_changed
+            self.inlet_flowrate_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_outletflowrate,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.outlet_flowrate_changed
+            self.outlet_flowrate_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.gb_inletpressure,
             QtCore.SIGNAL("clicked(bool)"),
-            self.inlet_pressure_active
+            self.inlet_pressure_active,
         )
         QtCore.QObject.connect(
             self.parameterWidget.gb_outletpressure,
             QtCore.SIGNAL("clicked(bool)"),
-            self.outlet_pressure_active
+            self.outlet_pressure_active,
         )
         QtCore.QObject.connect(
             self.parameterWidget.gb_inletflowrate,
             QtCore.SIGNAL("clicked(bool)"),
-            self.inlet_flowrate_active
+            self.inlet_flowrate_active,
         )
         QtCore.QObject.connect(
             self.parameterWidget.gb_outletflowrate,
             QtCore.SIGNAL("clicked(bool)"),
-            self.outlet_flowrate_active
+            self.outlet_flowrate_active,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_entrance_pipe_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.entrance_pipe_area_changed
+            self.entrance_pipe_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_entrance_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.entrance_area_changed
+            self.entrance_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_diaphragm_pipe_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.diaphragm_pipe_area_changed
+            self.diaphragm_pipe_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_diaphragm_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.diaphragm_area_changed
+            self.diaphragm_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_bend_pipe_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.bend_pipe_area_changed
+            self.bend_pipe_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.sb_bradius_pdiameter,
             QtCore.SIGNAL("valueChanged(double)"),
-            self.bradius_pdiameter_changed
+            self.bradius_pdiameter_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.sb_bend_angle,
             QtCore.SIGNAL("valueChanged(double)"),
-            self.bend_angle_changed
+            self.bend_angle_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.sb_bend_loss_coefficient,
             QtCore.SIGNAL("valueChanged(double)"),
-            self.bend_loss_coefficient_changed
+            self.bend_loss_coefficient_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_gatevalve_pipe_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.gatevalve_pipe_area_changed
+            self.gatevalve_pipe_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.sb_gatevalve_closing_coeff,
             QtCore.SIGNAL("valueChanged(double)"),
-            self.gatevalve_closing_coeff_changed
+            self.gatevalve_closing_coeff_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_colebrooke_pipe_area,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.colebrooke_pipe_area_changed
+            self.colebrooke_pipe_area_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_colebrooke_radius,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.colebrooke_radius_changed
+            self.colebrooke_radius_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.if_colebrooke_grain_diameter,
             QtCore.SIGNAL("valueChanged(Base::Quantity)"),
-            self.colebrooke_grain_diameter_changed
+            self.colebrooke_grain_diameter_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.sb_colebrooke_form_factor,
             QtCore.SIGNAL("valueChanged(double)"),
-            self.colebrooke_form_factor_changed
+            self.colebrooke_form_factor_changed,
         )
         QtCore.QObject.connect(
             self.parameterWidget.tw_pump_characteristics,
             QtCore.SIGNAL("cellChanged(int, int)"),
-            self.pump_characteristics_changed
+            self.pump_characteristics_changed,
         )
         # some fluid types deactivated since they are not implemented in ccx writer
         self.parameterWidget.cb_section_type.addItems(
@@ -233,10 +233,7 @@ class _TaskPanel:
 
         # geometry selection widget
         self.selectionWidget = selection_widgets.GeometryElementsSelection(
-            obj.References,
-            ["Edge"],
-            False,
-            True
+            obj.References, ["Edge"], False, True
         )
 
         # form made from param and selection widget
@@ -245,20 +242,12 @@ class _TaskPanel:
     def accept(self):
         self.set_fluidsection_props()
         self.obj.References = self.selectionWidget.references
-        self.recompute_and_set_back_all()
-        return True
+        self.selectionWidget.finish_selection()
+        return super().accept()
 
     def reject(self):
-        self.recompute_and_set_back_all()
-        return True
-
-    def recompute_and_set_back_all(self):
-        doc = FreeCADGui.getDocument(self.obj.Document)
-        doc.Document.recompute()
-        self.selectionWidget.setback_listobj_visibility()
-        if self.selectionWidget.sel_server:
-            FreeCADGui.Selection.removeObserver(self.selectionWidget.sel_server)
-        doc.resetEdit()
+        self.selectionWidget.finish_selection()
+        return super().reject()
 
     def get_fluidsection_props(self):
         self.SectionType = self.obj.SectionType
@@ -347,11 +336,11 @@ class _TaskPanel:
         self.parameterWidget.if_enlarge_area2.setText(self.EnlargeArea2.UserString)
         self.parameterWidget.if_contract_area1.setText(self.ContractArea1.UserString)
         self.parameterWidget.if_contract_area2.setText(self.ContractArea2.UserString)
-        self.parameterWidget.if_inletpressure.setText(FreeCAD.Units.Quantity(
-            1000 * self.InletPressure, FreeCAD.Units.Pressure).UserString
+        self.parameterWidget.if_inletpressure.setText(
+            FreeCAD.Units.Quantity(1000 * self.InletPressure, FreeCAD.Units.Pressure).UserString
         )
-        self.parameterWidget.if_outletpressure.setText(FreeCAD.Units.Quantity(
-            1000 * self.OutletPressure, FreeCAD.Units.Pressure).UserString
+        self.parameterWidget.if_outletpressure.setText(
+            FreeCAD.Units.Quantity(1000 * self.OutletPressure, FreeCAD.Units.Pressure).UserString
         )
         self.parameterWidget.if_inletflowrate.setText(str(self.InletFlowRate))
         self.parameterWidget.if_outletflowrate.setText(str(self.OutletFlowRate))
@@ -397,9 +386,7 @@ class _TaskPanel:
         self.parameterWidget.cb_liquid_section_type.setCurrentIndex(index)
         self.parameterWidget.sw_liquid_section_type.setCurrentIndex(index)
         # parameterWidget returns unicode
-        self.LiquidSectionType = str(
-            self.parameterWidget.cb_liquid_section_type.itemText(index)
-        )
+        self.LiquidSectionType = str(self.parameterWidget.cb_liquid_section_type.itemText(index))
 
     def manning_area_changed(self, base_quantity_value):
         self.ManningArea = base_quantity_value
